%-------------------------------------------------------------------------
% MATLAB code for Hennequin & Hommes (2022)
% "Managing bubbles in experimental asset markets with monetary policy”
%-------------------------------------------------------------------------

% import group data from Excel
  marketprice = xlsread('policyLtFE_allmarkets.xlsx', 'Price');
  interest = xlsread('policyLtFE_allmarkets.xlsx', 'Interest');
  prediction = xlsread('policyLtFE_allmarkets.xlsx', 'Prediction');

% number of markets
  num_markets = size(marketprice, 2);  
  
% group data by treatment
  T1_price = marketprice(:,1:8);
  T1_interest = interest(:,1:8);
  T1_predict = prediction(:,1:48);
  T2_price = marketprice(:,9:16);
  T2_interest = interest(:,9:16);
  T2_predict = prediction(:,49:96);
  T3_price = marketprice(:,17:24);
  T3_interest = interest(:,17:24);
  T3_predict = prediction(:,97:144);
  T4_price = marketprice(:,25:32);
  T4_interest = interest(:,25:32);
  T4_predict = prediction(:,145:192);
  T5_price = marketprice(:,33:40);
  T5_interest = interest(:,33:40);  
  T5_predict = prediction(:,193:240);

%-------------------------------------------------------------------------
% FIGURES PER TREATMENT (Figure 2 in paper)
%-------------------------------------------------------------------------

% Parameters for improved figures
  width = 3.5;              % Width in inches
  height = 1.5;             % Height in inches
  alw = 0.75;               % AxesLineWidth
  fsz = 11;                 % Fontsize
  lw = 1.5;                 % LineWidth
  msz = 3;                  % MarkerSize
  color = [0 0.75 0.75];    % 8th color
  
% Treatment Weak Rule
  % Market prices
  figure(1) 
  pos = get(gcf, 'Position');
  set(gcf, 'Position', [pos(1) pos(2) width*150, height*200]);
  set(gca, 'FontSize', fsz, 'LineWidth', alw);
  plot(T1_price(:,1:7),'LineWidth',lw,'MarkerSize',msz)
  hold on
  plot(T1_price(:,8),'LineWidth',lw,'MarkerSize',msz,'Color',color)
  hold on
  plot([0 50],[60 60], '--k','LineWidth',lw)
  hold off
  axis([0 50 0 1000])
  set(gca,'XTick',0:10:50,'YTick',0:250:1000)
  ylabel('Price', 'FontSize', fsz)
  % Interest rates
  figure(2) 
  pos = get(gcf, 'Position');
  set(gcf, 'Position', [pos(1) pos(2) width*150, height*150]);
  set(gca, 'FontSize', fsz, 'LineWidth', alw);
  plot(T1_interest(:,1:7),'LineWidth',lw,'MarkerSize',msz)
  hold on
  plot(T1_interest(:,8),'LineWidth',lw,'MarkerSize',msz,'Color',color)
  hold on
  plot([0 50],[5 5], '--k','LineWidth',lw)
  hold off
  axis([0 50 0 10])
  set(gca,'XTick',0:10:50,'YTick',0:2.5:10)
  ylabel('Interest rate (%)', 'FontSize', fsz)  

% Treatment Strong Rule
  % Market prices
  figure(3) 
  pos = get(gcf, 'Position');
  set(gcf, 'Position', [pos(1) pos(2) width*150, height*200]);
  set(gca, 'FontSize', fsz, 'LineWidth', alw);
  plot(T2_price(:,1:7),'LineWidth',lw,'MarkerSize',msz)
  hold on
  plot(T2_price(:,8),'LineWidth',lw,'MarkerSize',msz,'Color',color)
  hold on
  plot([0 50],[60 60], '--k','LineWidth',lw)
  hold off
  axis([0 50 0 500])
  set(gca,'XTick',0:10:50,'YTick',0:100:500)
  ylabel('Price')  
  % Interest rates
  figure(4) 
  pos = get(gcf, 'Position');
  set(gcf, 'Position', [pos(1) pos(2) width*150, height*150]);
  set(gca, 'FontSize', fsz, 'LineWidth', alw);
  plot(T2_interest(:,1:7),'LineWidth',lw,'MarkerSize',msz)
  hold on
  plot(T2_interest(:,8),'LineWidth',lw,'MarkerSize',msz,'Color',color)
  hold on 
  plot([0 50],[5 5], '--k','LineWidth',lw)
  hold off
  axis([0 50 0 75])
  set(gca,'XTick',0:10:50,'YTick',0:25:75)
  ylabel('Interest rate (%)', 'FontSize', fsz)  
  
% Treatment No Information
  % Market prices
  figure(5) 
  pos = get(gcf, 'Position');
  set(gcf, 'Position', [pos(1) pos(2) width*150, height*200]);
  set(gca, 'FontSize', fsz, 'LineWidth', alw);
  plot(T3_price(:,1:7),'LineWidth',lw,'MarkerSize',msz)
  hold on
  plot(T3_price(:,8),'LineWidth',lw,'MarkerSize',msz,'Color',color)
  hold on  
  plot([0 50],[60 60], '--k','LineWidth',lw)
  hold off
  axis([0 50 0 500])
  set(gca,'XTick',0:10:50,'YTick',0:100:500)
  ylabel('Price')  
  % Interest rates
  figure(6) 
  pos = get(gcf, 'Position');
  set(gcf, 'Position', [pos(1) pos(2) width*150, height*150]);
  set(gca, 'FontSize', fsz, 'LineWidth', alw);
  plot(T3_interest(:,1:7),'LineWidth',lw,'MarkerSize',msz)
  hold on
  plot(T3_interest(:,8),'LineWidth',lw,'MarkerSize',msz,'Color',color)
  hold on 
  plot([0 50],[5 5], '--k','LineWidth',lw)
  hold off
  axis([0 50 0 75])
  set(gca,'XTick',0:10:50,'YTick',0:25:75)
  ylabel('Interest rate (%)', 'FontSize', fsz)  
  
% Treatment Communication
  % Market prices
  figure(7) 
  pos = get(gcf, 'Position');
  set(gcf, 'Position', [pos(1) pos(2) width*150, height*200]);
  set(gca, 'FontSize', fsz, 'LineWidth', alw);
  plot(T4_price(:,1:7),'LineWidth',lw,'MarkerSize',msz)
  hold on
  plot(T4_price(:,8),'LineWidth',lw,'MarkerSize',msz,'Color',color)
  hold on 
  plot([0 50],[60 60], '--k','LineWidth',lw)
  hold off
  axis([0 50 0 500])
  set(gca,'XTick',0:10:50,'YTick',0:100:500)
  ylabel('Price', 'FontSize', fsz)  
  % Interest rates
  figure(8) 
  pos = get(gcf, 'Position');
  set(gcf, 'Position', [pos(1) pos(2) width*150, height*150]);
  set(gca, 'FontSize', fsz, 'LineWidth', alw);
  plot(T4_interest(:,1:7),'LineWidth',lw,'MarkerSize',msz)
  hold on
  plot(T4_interest(:,8),'LineWidth',lw,'MarkerSize',msz,'Color',color)
  hold on
  plot([0 50],[5 5], '--k','LineWidth',lw)
  hold off
  axis([0 50 0 75])
  set(gca,'XTick',0:10:50,'YTick',0:25:75)
  ylabel('Interest rate (%)', 'FontSize', fsz)  

% Treatment Sample Average
  % Market prices
  figure(9) 
  pos = get(gcf, 'Position');
  set(gcf, 'Position', [pos(1) pos(2) width*150, height*200]);
  set(gca, 'FontSize', fsz, 'LineWidth', alw);
  plot(T5_price(:,1:7),'LineWidth',lw,'MarkerSize',msz)
  hold on
  plot(T5_price(:,8),'LineWidth',lw,'MarkerSize',msz,'Color',color)
  hold on 
  plot([0 50],[60 60], '--k','LineWidth',lw)
  hold off
  axis([0 50 0 1000])
  set(gca,'XTick',0:10:50,'YTick',0:250:1000)
  ylabel('Price')  
  % Interest rates
  figure(10) 
  pos = get(gcf, 'Position');
  set(gcf, 'Position', [pos(1) pos(2) width*150, height*150]);
  set(gca, 'FontSize', fsz, 'LineWidth', alw);
  plot(T5_interest(:,1:7),'LineWidth',lw,'MarkerSize',msz)
  hold on
  plot(T5_interest(:,8),'LineWidth',lw,'MarkerSize',msz,'Color',color)
  hold on
  plot([0 50],[5 5], '--k','LineWidth',lw)
  hold off
  axis([0 50 0 75])
  set(gca,'XTick',0:10:50,'YTick',0:25:75)
  ylabel('Interest rate (%)', 'FontSize', fsz)  

%-------------------------------------------------------------------------
% BUBBLE MEASURES: RAD and RD (Figure 3 in paper)
%-------------------------------------------------------------------------

% calculate deviations from steady state price
  dev = (marketprice - 60)/60;
  absdev = abs(dev);
  
% calculate RD and RAD for all markets
  RD = NaN(num_markets,1);
  RAD = NaN(num_markets,1);
  for i=1:num_markets
      RD(i) = mean(dev(:,i));
      RAD(i) = mean(absdev(:,i));
  end
 
% group RAD and RD by treatment  
  T1_RAD = RAD(1:8);
  T1_RD = RD(1:8);
  T2_RAD = RAD(9:16);
  T2_RD = RD(9:16);
  T3_RAD = RAD(17:24);
  T3_RD = RD(17:24);
  T4_RAD = RAD(25:32);
  T4_RD = RD(25:32);
  T5_RAD = RAD(33:40);
  T5_RD = RD(33:40);

% parameters for improved figures
  width = 3.5;            % Width in inches
  height = 1.5;           % Height in inches
  alw = 0.75;             % AxesLineWidth
  fsz = 11;               % Fontsize
  lw = 2;                 % LineWidth

% plot CDFs of RAD
  figure(11)
  pos = get(gcf, 'Position');
  set(gcf, 'Position', [pos(1) pos(2) width*100, height*150]);
  set(gca, 'FontSize', fsz, 'LineWidth', alw);
  cdf1 = cdfplot(T1_RAD);
  hold on
  cdf2 = cdfplot(T2_RAD);
  hold on
  cdf3 = cdfplot(T3_RAD);
  hold on
  cdf4 = cdfplot(T4_RAD);
  hold on
  cdf5 = cdfplot(T5_RAD);
  hold off
  set(cdf1,'LineWidth',lw, 'LineStyle','-.')
  set(cdf2,'LineWidth',lw, 'LineStyle',':')
  set(cdf3,'LineWidth',lw)
  set(cdf4,'LineWidth',lw)
  set(cdf5,'LineWidth',lw, 'LineStyle','--')
  xlabel('RAD', 'FontSize', fsz)  
  ylabel('CDF', 'FontSize', fsz)  
  title('');
  legend('WR','SR','NI','C','SA','location','southeast')
  
% plot CDFs of RD
  figure(12)
  pos = get(gcf, 'Position');
  set(gcf, 'Position', [pos(1) pos(2) width*100, height*150]);
  set(gca, 'FontSize', fsz, 'LineWidth', alw);
  cdf1 = cdfplot(T1_RD);
  hold on
  cdf2 = cdfplot(T2_RD);
  hold on
  cdf3 = cdfplot(T3_RD);
  hold on
  cdf4 = cdfplot(T4_RD);
  hold on
  cdf5 = cdfplot(T5_RD);
  hold off
  set(cdf1,'LineWidth',lw, 'LineStyle','-.')
  set(cdf2,'LineWidth',lw, 'LineStyle',':')
  set(cdf3,'LineWidth',lw)
  set(cdf4,'LineWidth',lw)
  set(cdf5,'LineWidth',lw, 'LineStyle','--')
  xlabel('RD', 'FontSize', fsz)  
  ylabel('CDF', 'FontSize', fsz)  
  title('');
  legend('WR','SR','NI','C','SA','location','southeast')
  
% see R code for significance tests

%-------------------------------------------------------------------------
% QUADRATIC FORECAST ERRORS (Figure 4 in paper)
%-------------------------------------------------------------------------

% number of observations T (t=(t0+1):50 to allow for learning)
  t0 = 5;
  T = 50-t0;

% calculate errors for all markets
  avindividualerror = NaN(num_markets,1);
  avdispersionerror = NaN(num_markets,1);
  avcommonerror = NaN(num_markets,1);
  
  index_predict = 1:6:num_markets*6;
  for i=1:num_markets
      % average individual error
      avqfe = zeros(6,1);
      k = 1;
      for j=index_predict(i):index_predict(i)+5
          % quadratic forecast error
          qfe = (prediction((t0+1):50,j)-marketprice((t0+1):50,i)).^2;
          % average quadratic forecast error for participant i
          avqfe(k) = mean(qfe);
          k = k+1;
      end
      avindividualerror(i) = mean(avqfe);
      % average prediction in each period
      avpred = mean(prediction(:,index_predict(i):index_predict(i)+5),2);
      % average dispersion error
      avdisper = zeros(6,1);
      k = 1;
      for j=index_predict(i):index_predict(i)+5
          % quadratic forecast error
          disper = (prediction((t0+1):50,j)-avpred((t0+1):50)).^2;
          % average quadratic forecast error for participant i
          avdisper(k) = mean(disper);
          k = k+1;
      end
      avdispersionerror(i) = mean(avdisper);  
      % average common error
      comer = (avpred((t0+1):50)-marketprice((t0+1):50,i)).^2;
      avcommonerror(i) = mean(comer);
  end
  
% group qfe by treatment  
  T1_qfe = [avindividualerror(1:8)   avdispersionerror(1:8)    avcommonerror(1:8)];
  T2_qfe = [avindividualerror(9:16)  avdispersionerror(9:16)   avcommonerror(9:16)];
  T3_qfe = [avindividualerror(17:24) avdispersionerror(17:24)  avcommonerror(17:24)];
  T4_qfe = [avindividualerror(25:32) avdispersionerror(25:32)  avcommonerror(25:32)];
  T5_qfe = [avindividualerror(33:40) avdispersionerror(33:40)  avcommonerror(33:40)];  
  
% add percentages of dispersion error and common error
  T1_qfe(:,4:5) = [T1_qfe(:,2)./T1_qfe(:,1) T1_qfe(:,3)./T1_qfe(:,1)];
  T2_qfe(:,4:5) = [T2_qfe(:,2)./T2_qfe(:,1) T2_qfe(:,3)./T2_qfe(:,1)];
  T3_qfe(:,4:5) = [T3_qfe(:,2)./T3_qfe(:,1) T3_qfe(:,3)./T3_qfe(:,1)];
  T4_qfe(:,4:5) = [T4_qfe(:,2)./T4_qfe(:,1) T4_qfe(:,3)./T4_qfe(:,1)];
  T5_qfe(:,4:5) = [T5_qfe(:,2)./T5_qfe(:,1) T5_qfe(:,3)./T5_qfe(:,1)];    
  
% parameters for improved figures
  width = 3.5;            % Width in inches
  height = 1.5;           % Height in inches
  alw = 0.75;             % AxesLineWidth
  fsz = 11;               % Fontsize
  msz = 3;                % MarkerSize 
  lw = 2;                 % LineWidth  
  
% plot CDFs of avindividualerror
  figure(13)
  pos = get(gcf, 'Position');
  set(gcf, 'Position', [pos(1) pos(2) width*100, height*150]);
  set(gca, 'FontSize', fsz, 'LineWidth', alw);
  cdf1 = cdfplot(T1_qfe(:,1));
  hold on
  cdf2 = cdfplot(T2_qfe(:,1));
  hold on
  cdf3 = cdfplot(T3_qfe(:,1));
  hold on
  cdf4 = cdfplot(T4_qfe(:,1));
  hold on
  cdf5 = cdfplot(T5_qfe(:,1));
  hold off
  set(cdf1,'LineWidth',lw, 'LineStyle','-.')
  set(cdf2,'LineWidth',lw, 'LineStyle',':')
  set(cdf3,'LineWidth',lw)
  set(cdf4,'LineWidth',lw)
  set(cdf5,'LineWidth',lw, 'LineStyle','--')
  xlabel('average individual error', 'FontSize', fsz)  
  ylabel('CDF', 'FontSize', fsz)  
  title('');
  legend('WR','SR','NI','C','SA','location','southeast')  
  
  % plot CDFs of avdispersionerror
  figure(14)
  pos = get(gcf, 'Position');
  set(gcf, 'Position', [pos(1) pos(2) width*100, height*150]);
  set(gca, 'FontSize', fsz, 'LineWidth', alw);
  cdf1 = cdfplot(T1_qfe(:,4));
  hold on
  cdf2 = cdfplot(T2_qfe(:,4));
  hold on
  cdf3 = cdfplot(T3_qfe(:,4));
  hold on
  cdf4 = cdfplot(T4_qfe(:,4));
  hold on
  cdf5 = cdfplot(T5_qfe(:,4));
  hold off
  set(cdf1,'LineWidth',lw, 'LineStyle','-.')
  set(cdf2,'LineWidth',lw, 'LineStyle',':')
  set(cdf3,'LineWidth',lw)
  set(cdf4,'LineWidth',lw)
  set(cdf5,'LineWidth',lw, 'LineStyle','--')
  xlabel('% average dispersion error', 'FontSize', fsz)  
  ylabel('CDF', 'FontSize', fsz)  
  title('');
  legend('WR','SR','NI','C','SA','location','southeast')

%-------------------------------------------------------------------------
% STD AND CV OF PREDICTIONS (Figure 5 in paper)
%-------------------------------------------------------------------------

% calculate std and cv in each period at the group level 
  std_predict = NaN(51,num_markets);
  cv_predict = NaN(51,num_markets);
  index_predict = 1:6:num_markets*6;
  for t=1:51
      for i=1:num_markets
          std_predict(t,i) = std(prediction(t,index_predict(i):index_predict(i)+5));
          cv_predict(t,i) = std_predict(t,i)/mean(prediction(t,index_predict(i):index_predict(i)+5));
      end
  end
  
% group std_predict and cv_predict by treatment  
  T1_std = std_predict(:,1:8);
  T2_std = std_predict(:,9:16);
  T3_std = std_predict(:,17:24);
  T4_std = std_predict(:,25:32);
  T5_std = std_predict(:,33:40);
  T1_cv = cv_predict(:,1:8);
  T2_cv = cv_predict(:,9:16);
  T3_cv = cv_predict(:,17:24);
  T4_cv = cv_predict(:,25:32);
  T5_cv = cv_predict(:,33:40);

% calculate median std and cv for each treatment
  median_std = NaN(51,5);
  median_std(:,1) = median(T1_std,2);
  median_std(:,2) = median(T2_std,2);
  median_std(:,3) = median(T3_std,2);
  median_std(:,4) = median(T4_std,2);
  median_std(:,5) = median(T5_std,2);
  median_cv = NaN(51,5);
  median_cv(:,1) = median(T1_cv,2);
  median_cv(:,2) = median(T2_cv,2);
  median_cv(:,3) = median(T3_cv,2);
  median_cv(:,4) = median(T4_cv,2);
  median_cv(:,5) = median(T5_cv,2);

% parameters for improved figures
  width = 3.5;            % Width in inches
  height = 1.5;           % Height in inches
  alw = 0.75;             % AxesLineWidth
  fsz = 11;               % Fontsize
  msz = 3;                % MarkerSize 
  lw = 2;                 % LineWidth

% plot time series of median std for all treatments
  figure(15) 
  pos = get(gcf, 'Position');
  set(gcf, 'Position', [pos(1) pos(2) width*100, height*150]);
  set(gca, 'FontSize', fsz, 'LineWidth', alw);
  plot(median_std(:,1),'-.','LineWidth',lw,'MarkerSize',msz)
  hold on
  plot(median_std(:,2),':','LineWidth',lw,'MarkerSize',msz)
  hold on,
  plot(median_std(:,3),'LineWidth',lw,'MarkerSize',msz)
  hold on
  plot(median_std(:,4),'LineWidth',lw,'MarkerSize',msz)
  hold on
  plot(median_std(:,5),'--','LineWidth',lw,'MarkerSize',msz)
  hold off
  axis([0 51 0 50])
  set(gca,'XTick',0:10:51,'YTick',0:10:50)
  ylabel('median st.dev. of predictions', 'FontSize', fsz)  
  legend('WR','SR','NI','C','SA','location','northwest')
  
% plot time series of median cv for all treatments
  figure(16) 
  pos = get(gcf, 'Position');
  set(gcf, 'Position', [pos(1) pos(2) width*100, height*150]);
  set(gca, 'FontSize', fsz, 'LineWidth', alw);
  plot(median_cv(:,1),'-.','LineWidth',lw,'MarkerSize',msz)
  hold on
  plot(median_cv(:,2),':','LineWidth',lw,'MarkerSize',msz)
  hold on,
  plot(median_cv(:,3),'LineWidth',lw,'MarkerSize',msz)
  hold on
  plot(median_cv(:,4),'LineWidth',lw,'MarkerSize',msz)
  hold on
  plot(median_cv(:,5),'--','LineWidth',lw,'MarkerSize',msz)
  hold off
  axis([0 51 0 1])
  set(gca,'XTick',0:10:51,'YTick',0:0.2:1)
  ylabel('median CV of predictions', 'FontSize', fsz)  
  legend('WR','SR','NI','C','SA','location','northeast')