#------------------------------------------------------------------------------
# R code for Hennequin & Hommes (2022)
# “Managing bubbles in experimental asset markets with monetary policy”
#------------------------------------------------------------------------------

# clear workspace
  rm(list=ls(all=TRUE))

# load required packages
  require(readxl)
  require(lmtest)
  require(car)

# import data from Excel
  market <- read_excel("policyLtFE_allmarkets_outliersremoved.xlsx", sheet="Price")                           
  interest <- read_excel("policyLtFE_allmarkets_outliersremoved.xlsx", sheet="Interest")                   
  predict <- read_excel("policyLtFE_allmarkets_outliersremoved.xlsx", sheet="Prediction")                      

# number of participants and observations
  Np <- ncol(predict)
  Nobs <- nrow(predict)

# create matrices for results
  regressorsSLR <- matrix(0,nrow=8,ncol=Np)
  coeffSLR <- matrix(NA,nrow=9,ncol=Np)
  coefftableSLR <- matrix(0,nrow=Np,ncol=9)
  pvalueSLR <- matrix(NA,nrow=9,ncol=Np)
  RsquaredSLR <- matrix(NA,nrow=Np,ncol=1)
  pBG <- matrix(NA,nrow=Np,ncol=1)
  # column names
  colnames(coefftableSLR) <- c("alpha", "beta_1","beta_2","beta_3","beta_4","gamma_0","gamma_1","gamma_2","gamma_3")
  colnames(RsquaredSLR) <- c("adj_R^2")
  colnames(pBG) <- c("pvalue_BG")

#------------------------------------------------------------------------------
# ESTIMATE PREDICTION STRATEGIES: SIMPLE LINEAR RULE (SLR)
#------------------------------------------------------------------------------

# loop for all participants
  for (i in 1:Np){
      # determine corresponding group
        group <- ((i-1) %/% 6)+1
      # create dependent variable and regressors for t=6:51
        pred <- as.matrix(predict[6:Nobs,i])
        market_lag2 <- market[4:(Nobs-2),group]
        market_lag3 <- market[3:(Nobs-3),group]
        market_lag4 <- market[2:(Nobs-4),group]
        market_lag5 <- market[1:(Nobs-5),group]
        pred_lag1 <- predict[5:(Nobs-1),i]
        pred_lag2 <- predict[4:(Nobs-2),i]
        pred_lag3 <- predict[3:(Nobs-3),i]
        pred_lag4 <- predict[2:(Nobs-4),i]
        # create matrix of regressors X
        regressors <- c(1,2,3,4,5,6,7,8)
        X <- data.frame(market_lag2,market_lag3,market_lag4,market_lag5,pred_lag1,pred_lag2,pred_lag3,pred_lag4)
        colnames(X) <- c("market_lag2","market_lag3","market_lag4","market_lag5","pred_lag1","pred_lag2","pred_lag3","pred_lag4")
      # estimate model pred~1+X by OLS
        model <- lm(pred ~ ., data=X)
      # store coefficients and p-values using White standard errors
        coeff <- coef(model)
        pvalue <- coeftest(model,vcoc. = vcofHC)[,4]

      # remove insignificant regressors step by step
        for (k in 1:8){
         if  ((max(pvalue[-1]) > 0.05) && (pvalue[k+1] == max(pvalue[-1]))){
             regressors <- regressors[-k]
             X <- X[,-k]
             model <- lm(pred ~ ., data=X)
             coeff <- coef(model)
             pvalue <- coeftest(model,vcoc. = vcofHC)[,4]
             for (l in 1:7){
              if  ((max(pvalue[-1]) > 0.05) && (pvalue[l+1] == max(pvalue[-1]))){
                  regressors <- regressors[-l]
                  X <- X[,-l]
                  model <- lm(pred ~ ., data=X)
                  coeff <- coef(model)
                  pvalue <- coeftest(model,vcoc. = vcofHC)[,4]
                  for (m in 1:6){
                   if  ((max(pvalue[-1]) > 0.05) && (pvalue[m+1] == max(pvalue[-1]))){
                       regressors <- regressors[-m]
                       X <- X[,-m]
         		           model <- lm(pred ~ ., data=X)
            	         coeff <- coef(model)
            	         pvalue <- coeftest(model,vcoc. = vcofHC)[,4]
                       for (n in 1:5){
                        if  ((max(pvalue[-1]) > 0.05) && (pvalue[n+1] == max(pvalue[-1]))){
                            regressors <- regressors[-n]
                            X <- X[,-n]
                            model <- lm(pred ~ ., data=X)
             	              coeff <- coef(model)
            		            pvalue <- coeftest(model,vcoc. = vcofHC)[,4]
                            for (p in 1:4){
                             if  ((max(pvalue[-1]) > 0.05) && (pvalue[p+1] == max(pvalue[-1]))){
                                 regressors <- regressors[-p]
                                 X <- X[,-p]
        			                   model <- lm(pred ~ ., data=X)
      			                     coeff <- coef(model)
          				               pvalue <- coeftest(model,vcoc. = vcofHC)[,4]
                                 for (q in 1:3){
                                  if  ((max(pvalue[-1]) > 0.05) && (pvalue[q+1] == max(pvalue[-1]))){
                                      regressors <- regressors[-q]
                                      X <- X[,-q]
           					                  model <- lm(pred ~ ., data=X)
          				                    coeff <- coef(model)
          				                    pvalue <- coeftest(model,vcoc. = vcofHC)[,4]
                                      for (r in 1:2){
                                       if  ((max(pvalue[-1]) > 0.05) && (pvalue[r+1] == max(pvalue[-1]))){
                                           regressors <- regressors[-r]
                                           model <- lm(pred ~ ., data=X[-r])
                                           X <- X[,-r]
          					                       coeff <- coef(model)
           					                       pvalue <- coeftest(model,vcoc. = vcofHC)[,4]
                                           if  (pvalue[2] > 0.05){
                                               regressors <- 0
                                               X <- {}
                                               model <- lm(pred ~ 1)
             					                         coeff <- coef(model)
             					                         pvalue <- coeftest(model,vcoc. = vcofHC)[,4]
                                           }
                                       }
                                      }
                                  }
                                 }
                             }
                            }
                        }
                       }
                   }
                  }
              }
             }
         }
        }
      # save regressors, coefficients, p-values and R^2
        regressorsSLR[1:length(regressors),i] <- regressors
        coeffSLR[1:length(coeff),i] <- coeff
        pvalueSLR[1:length(pvalue),i] <- pvalue
        RsquaredSLR[i] <- summary(model)$adj.r.squared
      # make table of coefficients SLR
        coefftableSLR[i,1] <- coeffSLR[1,i];
        for (row in 1:8){
            for (value in 1:8){
                if  (regressorsSLR[row,i] == value){
                    coefftableSLR[i,value+1] <- coeffSLR[row+1,i]
                }
            }
        }

      # diagnostic tests (only if matrix of regressors X is not empty)
        if (!is.null(X)){
           # Breusch-Godfrey test for autocorrelation (2 lags)
           BG <- bgtest(model, order=2, type="Chisq")
           pBG[i] <- BG$p.value
        }
        
  } #close loop for al participants

# write results SLR to table
  resultsSLR <- cbind(coefftableSLR, RsquaredSLR, pBG)

#------------------------------------------------------------------------------
# MAIN RESULTS (Table 3 in paper)
#------------------------------------------------------------------------------

# create empty table
  mainresults <- matrix(NA,nrow=19,ncol=6)
  colnames(mainresults) <- c("WR","SR","NI","C","SA","Overall")
  rownames(mainresults) <- c("% success","mean R^2",
                             "% used beta_1","% used beta_2","% used beta_3","% used beta_4",
                             "% used gamma_0","% used gamma_1","% used gamma_2","% used gamma_3",
                             "mean alpha", "mean beta_1","mean beta_2","mean beta_3","mean beta_4",
                             "mean gamma_0","mean gamma_1","mean gamma_2","mean gamma_3")
  
# main results per treatment and overall
  for (t in 1:6){
      # subset of successful rules
      if (t == 6){
         success <- subset(resultsSLR, pBG > 0.05)
      } else {
         success <- subset(resultsSLR[(t*48-47):(t*48),], pBG[(t*48-47):(t*48),] > 0.05)
      }
      # % successful
      if (t == 6){
         mainresults[1,t] <- (nrow(success)/240)*100
      } else {
         mainresults[1,t] <- (nrow(success)/48)*100
      }
      # mean adj. R-squared
      mainresults[2,t] <- mean(success[,10])
      # % used (nonzero coefficients)
      for (h in 2:9){
          mainresults[h+1,t] <- (sum(success[,h] != 0)/nrow(success))*100
      }
      # mean coefficients
      for (h in 1:9){
        success[success == 0] <- NA
        mainresults[h+10,t] <- mean(success[,h], na.rm=TRUE)
      }
  }

#------------------------------------------------------------------------------
# MWW TESTS FOR SIGNIFICANT DIFFERENCES BETWEEN TREATMENTS SR, NI and C
#------------------------------------------------------------------------------

# group by treatment
  coeffT2 <- resultsSLR[49:96,];
  coeffT3 <- resultsSLR[97:144,];
  coeffT4 <- resultsSLR[145:192,];

# remove unsuccessful rules
  coeffT2 <- subset(coeffT2, pBG[49:96] > 0.05)
  coeffT3 <- subset(coeffT3, pBG[97:144] > 0.05)
  coeffT4 <- subset(coeffT4, pBG[145:192] > 0.05)

# create matrix for results
  MWtest <- matrix(NA,nrow=9,ncol=3)
  colnames(MWtest) <- c("SR vs. NI","SR vs. C","NI vs. C")
  rownames(MWtest) <- c("alpha", "beta_1","beta_2","beta_3","beta_4","gamma_0","gamma_1","gamma_2","gamma_3")
  
# loop for all coefficients
  for (c in 1:9){
      # set test variables
      T2 <- coeffT2[,c]
      T3 <- coeffT3[,c]
      T4 <- coeffT4[,c]
      # MWW test coefficients
      MWtest[c,1] <- wilcox.test(coeffT2[,c], coeffT3[,c])$p.value
      MWtest[c,2] <- wilcox.test(coeffT2[,c], coeffT4[,c])$p.value
      MWtest[c,3] <- wilcox.test(coeffT3[,c], coeffT4[,c])$p.value
  }
