#------------------------------------------------------------------------------
# R code for Hennequin & Hommes (2022)
# “Managing bubbles in experimental asset markets with monetary policy”
#------------------------------------------------------------------------------

# clear workspace
rm(list=ls(all=TRUE))

# load required packages
require(readxl)

#------------------------------------------------------------------------------
# BUBBLE MEASURES (RAD and RD): PAIRWISE MWW TESTS
#------------------------------------------------------------------------------

# import data from Excel
bubbles <- read_excel("stats_for_MWW_testing.xlsx", sheet="Bubbles")

# group bubble measures by treatment
T1 <- bubbles[1:8,];
T2 <- bubbles[9:16,];
T3 <- bubbles[17:24,];
T4 <- bubbles[25:32,];
T5 <- bubbles[33:40,];

# create matrix for results
MWtest_RAD <- matrix(NA,nrow=5,ncol=5)
MWtest_RD <- matrix(NA,nrow=5,ncol=5)
colnames(MWtest_RAD) <- c("WR","SR","NI","C","SA")
rownames(MWtest_RAD) <- c("WR","SR","NI","C","SA")
colnames(MWtest_RD) <- c("WR","SR","NI","C","SA")
rownames(MWtest_RD) <- c("WR","SR","NI","C","SA")

# MWW test RAD: cell [i,j] gives the p-value for treatment i vs. treatment j
MWtest_RAD[1,2] <- wilcox.test(T1$RAD, T2$RAD)$p.value
MWtest_RAD[1,3] <- wilcox.test(T1$RAD, T3$RAD)$p.value
MWtest_RAD[1,4] <- wilcox.test(T1$RAD, T4$RAD)$p.value
MWtest_RAD[1,5] <- wilcox.test(T1$RAD, T5$RAD)$p.value
MWtest_RAD[2,3] <- wilcox.test(T2$RAD, T3$RAD)$p.value
MWtest_RAD[2,4] <- wilcox.test(T2$RAD, T4$RAD)$p.value
MWtest_RAD[2,5] <- wilcox.test(T2$RAD, T5$RAD)$p.value
MWtest_RAD[3,4] <- wilcox.test(T3$RAD, T4$RAD)$p.value
MWtest_RAD[3,5] <- wilcox.test(T3$RAD, T5$RAD)$p.value
MWtest_RAD[4,5] <- wilcox.test(T4$RAD, T5$RAD)$p.value

# MWW test RD: cell [i,j] gives the p-value for treatment i vs. treatment j
MWtest_RD[1,2] <- wilcox.test(T1$RD, T2$RD)$p.value
MWtest_RD[1,3] <- wilcox.test(T1$RD, T3$RD)$p.value
MWtest_RD[1,4] <- wilcox.test(T1$RD, T4$RD)$p.value
MWtest_RD[1,5] <- wilcox.test(T1$RD, T5$RD)$p.value
MWtest_RD[2,3] <- wilcox.test(T2$RD, T3$RD)$p.value
MWtest_RD[2,4] <- wilcox.test(T2$RD, T4$RD)$p.value
MWtest_RD[2,5] <- wilcox.test(T2$RD, T5$RD)$p.value
MWtest_RD[3,4] <- wilcox.test(T3$RD, T4$RD)$p.value
MWtest_RD[3,5] <- wilcox.test(T3$RD, T5$RD)$p.value
MWtest_RD[4,5] <- wilcox.test(T4$RD, T5$RD)$p.value

#------------------------------------------------------------------------------
# COORDINATION (QUADRATIC FORECAST ERROR): PAIRWISE MWW TESTS
#------------------------------------------------------------------------------

# import data from Excel
qfe <- read_excel("stats_for_MWW_testing.xlsx", sheet="Coordination")

# group qfe by treatment
qfeT1 <- qfe[1:8,];
qfeT2 <- qfe[9:16,];
qfeT3 <- qfe[17:24,];
qfeT4 <- qfe[25:32,];
qfeT5 <- qfe[33:40,];

# set test variable: average dispersion error
T1 <- qfeT1$`av.disp.err`
T2 <- qfeT2$`av.disp.err`
T3 <- qfeT3$`av.disp.err`
T4 <- qfeT4$`av.disp.err`
T5 <- qfeT5$`av.disp.err`

# create matrix for results
MWtest_qfe <- matrix(NA,nrow=5,ncol=5)
colnames(MWtest_qfe) <- c("WR","SR","NI","C","SA")
rownames(MWtest_qfe) <- c("WR","SR","NI","C","SA")

# MWW test qfe: cell [i,j] gives the p-value for treatment i vs. treatment j
MWtest_qfe[1,2] <- wilcox.test(T1, T2)$p.value
MWtest_qfe[1,3] <- wilcox.test(T1, T3)$p.value
MWtest_qfe[1,4] <- wilcox.test(T1, T4)$p.value
MWtest_qfe[1,5] <- wilcox.test(T1, T5)$p.value
MWtest_qfe[2,3] <- wilcox.test(T2, T3)$p.value
MWtest_qfe[2,4] <- wilcox.test(T2, T4)$p.value
MWtest_qfe[2,5] <- wilcox.test(T2, T5)$p.value
MWtest_qfe[3,4] <- wilcox.test(T3, T4)$p.value
MWtest_qfe[3,5] <- wilcox.test(T3, T5)$p.value
MWtest_qfe[4,5] <- wilcox.test(T4, T5)$p.value
